@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2003-2008 Free Software Foundation Europe e.V.
+   Copyright (C) 2003-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -31,7 +31,6 @@
  *
  *     Kern Sibbald, April MMIII
  *
- *   Version $Id$
  */
 
 #include "bacula.h"
@@ -157,6 +156,7 @@ static bool get_user_slot_list(UAContext *ua, char *slot_list, int num_slots)
    return true;
 
 bail_out:
+   Dmsg1(100, "Problem with user selection ERR=%s\n", msg);
    return false;
 }
 
@@ -1197,7 +1197,6 @@ void status_slots(UAContext *ua, STORE *store_r)
    MEDIA_DBR mr;
    char *slot_list;
    int max_slots;
-   int drive;
    int i=1;
    /* Slot | Volume | Status | MediaType | Pool */
    const char *slot_hformat=" %4i%c| %16s | %9s | %20s | %18s |\n";
@@ -1214,7 +1213,7 @@ void status_slots(UAContext *ua, STORE *store_r)
 
    pm_strcpy(store.store_source, _("command line"));
    set_wstorage(ua->jcr, &store);
-   drive = get_storage_drive(ua, store.store);
+   get_storage_drive(ua, store.store);
 
    max_slots = get_num_slots_from_SD(ua);
 
