@@ -1,7 +1,7 @@
 /*
-   Bacula® - The Network Backup Solution
+   Bacula(R) - The Network Backup Solution
 
-   Copyright (C) 2007-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -20,7 +20,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
 
-   Bacula® is a registered trademark of Kern Sibbald.
+   Bacula(R) is a registered trademark of Kern Sibbald.
    The licensor of Bacula is the Free Software Foundation Europe
    (FSFE), Fiduciary Program, Sumatrastrasse 25, 8006 Zürich,
    Switzerland, email:ftf@fsfeurope.org.
@@ -50,7 +50,7 @@ int readdir_r(DIR *dirp, struct dirent *entry, struct dirent **result);
 static const int dbglvl = 50;
 
 /* All loaded plugins */
-alist *plugin_list = NULL;
+alist *bplugin_list = NULL;
 
 /*
  * Create a new plugin "class" entry and enter it in the
@@ -101,6 +101,7 @@ bool load_plugins(void *binfo, void *bfuncs, const char *plugin_dir,
    int len, type_len;
 
 
+   Dmsg0(dbglvl, "load_plugins\n");
    name_max = pathconf(".", _PC_NAME_MAX);
    if (name_max < 1024) {
       name_max = 1024;
@@ -142,7 +143,7 @@ bool load_plugins(void *binfo, void *bfuncs, const char *plugin_dir,
          Dmsg3(dbglvl, "Rejected plugin: want=%s name=%s len=%d\n", type, result->d_name, len);
          continue;
       }
-      Dmsg2(dbglvl, "Loaded plugin: name=%s len=%d\n", result->d_name, len);
+      Dmsg2(dbglvl, "Found plugin: name=%s len=%d\n", result->d_name, len);
        
       pm_strcpy(fname, plugin_dir);
       if (need_slash) {
@@ -155,12 +156,14 @@ bool load_plugins(void *binfo, void *bfuncs, const char *plugin_dir,
 
       plugin = new_plugin();
       plugin->file = bstrdup(result->d_name);
+      plugin->file_len = strstr(plugin->file, type) - plugin->file;
       plugin->pHandle = dlopen(fname.c_str(), RTLD_NOW);
       if (!plugin->pHandle) {
-         Jmsg(NULL, M_ERROR, 0, _("Plugin load %s failed: ERR=%s\n"), 
-              fname.c_str(), NPRT(dlerror()));
-         Dmsg2(dbglvl, "Plugin load %s failed: ERR=%s\n", fname.c_str(), 
-               NPRT(dlerror()));
+         char *error = (char *)dlerror();
+         Jmsg(NULL, M_ERROR, 0, _("dlopen plugin %s failed: ERR=%s\n"), 
+              fname.c_str(), NPRT(error));
+         Dmsg2(dbglvl, "dlopen plugin %s failed: ERR=%s\n", fname.c_str(), 
+               NPRT(error));
          close_plugin(plugin);
          continue;
       }
@@ -198,7 +201,7 @@ bool load_plugins(void *binfo, void *bfuncs, const char *plugin_dir,
       }
 
       found = true;                /* found a plugin */
-      plugin_list->append(plugin);
+      bplugin_list->append(plugin);
    }
 
 get_out:
@@ -221,10 +224,10 @@ void unload_plugins()
 {
    Plugin *plugin;
 
-   if (!plugin_list) {
+   if (!bplugin_list) {
       return;
    }
-   foreach_alist(plugin, plugin_list) {
+   foreach_alist(plugin, bplugin_list) {
       /* Shut it down and unload it */
       plugin->unloadPlugin();
       dlclose(plugin->pHandle);
@@ -233,8 +236,8 @@ void unload_plugins()
       }
       free(plugin);
    }
-   delete plugin_list;
-   plugin_list = NULL;
+   delete bplugin_list;
+   bplugin_list = NULL;
 }
 
 /*
@@ -257,10 +260,10 @@ void dbg_print_plugin(FILE *fp)
    Plugin *plugin;
    fprintf(fp, "Attempt to dump plugins. Hook count=%d\n", dbg_plugin_hook_count);
 
-   if (!plugin_list) {
+   if (!bplugin_list) {
       return;
    }
-   foreach_alist(plugin, plugin_list) {
+   foreach_alist(plugin, bplugin_list) {
       for(int i=0; i < dbg_plugin_hook_count; i++) {
 //       dbg_plugin_hook_t *fct = dbg_plugin_hooks[i];
          fprintf(fp, "Plugin %p name=\"%s\" disabled=%d\n",
