@@ -38,7 +38,7 @@
 #include "run/run.h"
 #include "util/fmtwidgetitem.h"
 
-MediaView::MediaView()
+MediaView::MediaView() : Pages()
 {
    setupUi(this);
    m_name = tr("Media");
@@ -115,25 +115,37 @@ void MediaView::purgePushed()
 
 bool MediaView::getSelection(QStringList &list)
 {
-   QList<QTableWidgetItem*> items = m_tableMedia->selectedItems();
+   int i, nb, nr_rows, row;
+   bool *tab;
    QTableWidgetItem *it;
-   int row;
-   int *tab;
-   int nb = items.count();
+   QList<QTableWidgetItem*> items = m_tableMedia->selectedItems();
+
+   /*
+    * See if anything is selected.
+    */
+   nb = items.count();
    if (!nb) {
       return false;
    }
-   tab = (int *) malloc (nb * sizeof(int));
-   memset(tab, 0, sizeof(int)*nb);
-   for (int i = 0; i < nb; ++i) {
+
+   /*
+    * Create a nibble map for each row so we can see if its
+    * selected or not.
+    */
+   nr_rows = m_tableMedia->rowCount();
+   tab = (bool *)malloc (nr_rows * sizeof(bool));
+   memset(tab, 0, sizeof(bool) * nr_rows);
+
+   for (i = 0; i < nb; ++i) {
       row = items[i]->row();
       if (!tab[row]) {
-         tab[row]=1;
+         tab[row] = true;
          it = m_tableMedia->item(row, 0);
          list.append(it->text());
       }
    }
    free(tab);
+
    return list.count() > 0;
 }
 
@@ -324,7 +336,7 @@ void MediaView::populateTable()
 
       foreach (resultline, results) { // should have only one result
          int index = 0;
-         QString VolBytes, MediaType, LastWritten, VolStatus;
+         QString Slot, VolBytes, MediaType, LastWritten, VolStatus;
          fieldlist = resultline.split("\t");
          if (fieldlist.size() != 10) {
             continue;
@@ -337,7 +349,9 @@ void MediaView::populateTable()
          
          /* Online */
          mediaitem.setInChanger(index++, fld.next());
-         fld.next();            // Slot
+
+         Slot = fld.next();            // Slot
+         mediaitem.setTextFld(index++, Slot);
 
          MediaType = fld.next();
          VolStatus = fld.next();
