@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2007-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2007-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -31,20 +31,19 @@
  *
  *   Kern Sibbald, February MMVII
  *
- *  $Id$
  */ 
 
 #include "bat.h"
 #include "run.h"
 
 
-runPage::runPage()
+runPage::runPage() : Pages()
 {
    init();
    show();
 }
 
-runPage::runPage(const QString &defJob)
+runPage::runPage(const QString &defJob) : Pages()
 {
    m_dockOnFirstUse = false;
    init();
@@ -57,6 +56,7 @@ runPage::runPage(const QString &defJob)
 runPage::runPage(const QString &defJob, const QString &level, 
                  const QString &pool, const QString &storage,
                  const QString &client, const QString &fileset)
+   : Pages()
 {
    m_dockOnFirstUse = false;
    init();
@@ -82,10 +82,16 @@ runPage::runPage(const QString &defJob, const QString &level,
 void runPage::init()
 {
    QDateTime dt;
+   QDesktopWidget *desk = QApplication::desktop(); 
+   QRect scrn;
 
    m_name = tr("Run");
    pgInitialize();
    setupUi(this);
+   /* Get screen rectangle */
+   scrn = desk->screenGeometry(desk->primaryScreen());
+   /* Position this window in the middle of the screen */
+   this->move((scrn.width()-this->width())/2, (scrn.height()-this->height())/2);
    QTreeWidgetItem* thisitem = mainWin->getFromHash(this);
    thisitem->setIcon(0,QIcon(QString::fromUtf8(":images/run.png")));
    m_conn = m_console->notifyOff();
@@ -111,7 +117,7 @@ void runPage::init()
    connect(cancelButton, SIGNAL(pressed()), this, SLOT(cancelButtonPushed()));
 
    // find a way to place the new window at the cursor position
-   // or in the midle of the page
+   // or in the middle of the page
 //   dockPage();
    setCurrent();
 }
@@ -171,6 +177,7 @@ void runPage::job_name_change(int index)
    (void)index;
    job_defs.job_name = jobCombo->currentText();
    if (m_console->get_job_defaults(job_defs)) {
+      QString cmd;
       typeLabel->setText("<H3>"+job_defs.type+"</H3>");
       filesetCombo->setCurrentIndex(filesetCombo->findText(job_defs.fileset_name, Qt::MatchExactly));
       levelCombo->setCurrentIndex(levelCombo->findText(job_defs.level, Qt::MatchExactly));
@@ -178,5 +185,11 @@ void runPage::job_name_change(int index)
       poolCombo->setCurrentIndex(poolCombo->findText(job_defs.pool_name, Qt::MatchExactly));
       storageCombo->setCurrentIndex(storageCombo->findText(job_defs.store_name, Qt::MatchExactly));
       messagesCombo->setCurrentIndex(messagesCombo->findText(job_defs.messages_name, Qt::MatchExactly));
+      m_console->level_list.clear();
+      cmd = ".levels " + job_defs.type;
+      m_console->dir_cmd(cmd, m_console->level_list);
+      levelCombo->clear();
+      levelCombo->addItems(m_console->level_list);
+      levelCombo->setCurrentIndex(levelCombo->findText(job_defs.level, 0 /*Qt::MatchExactly*/));
    }
 }
