@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2007-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2007-2010 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -27,7 +27,6 @@
 */
  
 /*
- *   Version $Id$
  *
  *  Storage Class
  *
@@ -45,7 +44,7 @@
 #include "status/storstat.h"
 #include "util/fmtwidgetitem.h"
 
-Storage::Storage()
+Storage::Storage() : Pages()
 {
    setupUi(this);
    pgInitialize(tr("Storage"));
