@@ -1,7 +1,7 @@
 /*
    Bacula® - The Network Backup Solution
 
-   Copyright (C) 2000-2009 Free Software Foundation Europe e.V.
+   Copyright (C) 2000-2011 Free Software Foundation Europe e.V.
 
    The main author of Bacula is Kern Sibbald, with contributions from
    many others, a complete list can be found in the file AUTHORS.
@@ -72,6 +72,10 @@ const int sd_dbglvl = 300;
 #else
 #define uLongf uint32_t
 #endif
+#ifdef HAVE_LZO
+#include <lzo/lzoconf.h>
+#include <lzo/lzo1x.h>
+#endif
 #ifdef HAVE_FNMATCH
 #include <fnmatch.h>
 #else
@@ -86,6 +90,7 @@ int readdir_r(DIR *dirp, struct dirent *entry, struct dirent **result);
 #endif
 
 #include "vtape.h"
+#include "sd_plugins.h"
 
 /* Daemon globals from stored.c */
 extern STORES *me;                    /* "Global" daemon resource */
